package msoetestngdemo;
import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertNotNull;
import msoetestngdemo.BloodPressureDiagnosisSystem;

import org.testng.annotations.Test;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;

/**
 * This class will test the blood pressure system using testNG.
 * 
 * @author wws
 * 
 */

public class BloodPressureDiagnosisSystemTest {

	private BloodPressureDiagnosisSystem patient;

	@BeforeMethod
	/**
	 * This method will create a new instance of the patient before each test executes.
	 * @throws Exception
	 */
	public void setup() throws Exception {
		// Create a new valid patient.
		patient = new BloodPressureDiagnosisSystem("Ima Tester", 0x21);
	}

	@DataProvider(name = "constructorTests")
	public static Object[][] createConstructorData() {
		return new Object[][] { { "Janice Tester", new Integer(21) },
				{ "Janice Tester", new Integer(0) },
				{ "Janice Tester", new Integer(1) },
				{ "Ja Tester", new Integer(21) }, { "Ja Te", new Integer(21) } };
	}

	@Test(dataProvider = "constructorTests")
	/**
	 * This test will insure that the normal constructor works preoperly.
	 */
	public void testNormalConstructor(String name, int age) throws Exception {
		patient = new BloodPressureDiagnosisSystem(name, age);
		// Make sure the patient is not null.
		assertNotNull(patient);

		// Make sure the age is set correctly.
		assertEquals(age, patient.getAge());

		// Make sure the name is set correctly.
		assertEquals(name, patient.getPatientName());
	}

	@DataProvider(name = "invalidconstructorTests")
	public static Object[][] createInvalidConstructorData() {
		return new Object[][] { { "Janice Tester", new Integer(-1) },
				{ "J Tester", new Integer(21) }, { "Ja T", new Integer(21) },
				{ "JaT", new Integer(21) } };
	}

	@Test(dataProvider = "invalidconstructorTests", expectedExceptions = Exception.class)
	/**
	 * This test will ensure that a negative age fails properly.
	 */
	public void testNegativeAgeOutOfBoundsInConstructor(String name, int age)
			throws Exception {
		patient = new BloodPressureDiagnosisSystem(name, age);
	}

	@DataProvider(name = "validBPReadings")
	public static Object[][] createvalidBPData() {
		return new Object[][] {
				// Data sets for the normal to prehypertension transition.
				{
						new Integer(119),
						new Integer(79),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.NORMAL },
				{
						new Integer(119),
						new Integer(80),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },
				{
						new Integer(119),
						new Integer(81),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },
				{
						new Integer(120),
						new Integer(79),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },
				{
						new Integer(120),
						new Integer(80),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },
				{
						new Integer(120),
						new Integer(81),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },
				{
						new Integer(121),
						new Integer(79),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },
				{
						new Integer(121),
						new Integer(80),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },
				{
						new Integer(121),
						new Integer(81),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },

				// Datasets for the prehypertension to stage1 HBP testing.
				{
						new Integer(139),
						new Integer(89),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.PREHYPERTENSION },
				{
						new Integer(139),
						new Integer(90),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },
				{
						new Integer(139),
						new Integer(91),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },
				{
						new Integer(140),
						new Integer(89),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },
				{
						new Integer(140),
						new Integer(90),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },
				{
						new Integer(140),
						new Integer(91),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },
				{
						new Integer(141),
						new Integer(89),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },
				{
						new Integer(141),
						new Integer(90),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },
				{
						new Integer(141),
						new Integer(91),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },

				// Datasets for the stage1 to stage2 testing.
				{
						new Integer(159),
						new Integer(99),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_1_HBP },
				{
						new Integer(159),
						new Integer(100),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },
				{
						new Integer(159),
						new Integer(101),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },
				{
						new Integer(160),
						new Integer(99),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },
				{
						new Integer(160),
						new Integer(100),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },
				{
						new Integer(160),
						new Integer(101),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },
				{
						new Integer(161),
						new Integer(99),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },
				{
						new Integer(161),
						new Integer(100),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },
				{
						new Integer(161),
						new Integer(101),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },

				// Datasets for the hypertensive crisis testing.
				{
						new Integer(179),
						new Integer(109),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.STAGE_2_HBP },
				{
						new Integer(179),
						new Integer(110),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.HYPERTENSIVE_CRISIS },
				{
						new Integer(179),
						new Integer(111),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.HYPERTENSIVE_CRISIS },
				{
						new Integer(180),
						new Integer(109),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.HYPERTENSIVE_CRISIS },
				{
						new Integer(180),
						new Integer(110),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.HYPERTENSIVE_CRISIS },
				{
						new Integer(180),
						new Integer(111),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.HYPERTENSIVE_CRISIS },
				{
						new Integer(181),
						new Integer(109),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.HYPERTENSIVE_CRISIS },
				{
						new Integer(181),
						new Integer(110),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.HYPERTENSIVE_CRISIS },
				{
						new Integer(181),
						new Integer(111),
						BloodPressureDiagnosisSystem.BloodPressureDiagnosis.HYPERTENSIVE_CRISIS }, };
	}

	@Test(dataProvider = "validBPReadings")
	public void diagnoseBP(int systolic, int diastolic,
			BloodPressureDiagnosisSystem.BloodPressureDiagnosis diagnosis) {
		assertEquals(diagnosis, patient.diagnoseBP(systolic, diastolic));

	}
}
